@@ -258,8 +258,8 @@ class TreeMapper extends QBMapper {
 	 * @psalm-param T $type
 	 * @param int $folderId
 	 * @param bool $softDeleted
-	 * @return Entity[]
-	 * @psalm-return E[]
+	 * @return list<Entity>
+	 * @psalm-return list<E>
 	 * @psalm-template T as TreeMapper::TYPE_*
 	 * @psalm-template E as (T is TreeMapper::TYPE_FOLDER ? Folder : (T is TreeMapper::TYPE_BOOKMARK ? Bookmark : SharedFolder))
 	 */
@@ -313,21 +313,41 @@ class TreeMapper extends QBMapper {
 	 * @param string $type
 	 * @psalm-param T $type
 	 * @param int $folderId
-	 * @return Entity[]
-	 * @psalm-return E[]
+	 * @return list<Entity>
+	 * @psalm-return list<E>
 	 * @psalm-template T as TreeMapper::TYPE_*
 	 * @psalm-template E as (T is TreeMapper::TYPE_FOLDER ? Folder : (T is TreeMapper::TYPE_BOOKMARK ? Bookmark : SharedFolder))
 	 */
 	public function findByAncestorFolder(string $type, int $folderId): array {
-		$descendants = [];
-		$newDescendants = $this->findChildren($type, $folderId);
-		do {
-			array_push($descendants, ...$newDescendants);
-			$newDescendants = array_flatten(array_map(function (Entity $descendant) use ($type) {
-				return $this->findChildren($type, $descendant->getId());
-			}, $newDescendants));
-		} while (count($newDescendants) > 0);
-		return $descendants;
+		if ($type === TreeMapper::TYPE_FOLDER) {
+			$descendants = [];
+			$newDescendants = $this->findChildren(TreeMapper::TYPE_FOLDER, $folderId);
+			do {
+				array_push($descendants, ...$newDescendants);
+				$newDescendants = array_merge(...array_map(function (Folder $folder): array {
+					return $this->findChildren(TreeMapper::TYPE_FOLDER, $folder->getId());
+				}, $newDescendants));
+			} while (count($newDescendants) > 0);
+			return $descendants;
+		}
+
+		if ($type === TreeMapper::TYPE_SHARE) {
+			$descendantFolders = $this->findChildren(TreeMapper::TYPE_FOLDER, $folderId);
+			$descendants = array_merge(...array_map(function (Folder $folder): array {
+				return $this->findChildren(TreeMapper::TYPE_SHARE, $folder->getId());
+			}, $descendantFolders));
+			return $descendants;
+		}
+
+		if ($type === TreeMapper::TYPE_BOOKMARK) {
+			$descendantFolders = $this->findChildren(TreeMapper::TYPE_FOLDER, $folderId);
+			$descendants = array_merge(...array_map(function (Folder $folder): array {
+				return $this->findChildren(TreeMapper::TYPE_BOOKMARK, $folder->getId());
+			}, $descendantFolders));
+			return $descendants;
+		}
+
+		throw new \InvalidArgumentException('Invalid type');
 	}
 
 	/**
@@ -341,7 +361,7 @@ class TreeMapper extends QBMapper {
 		while (!in_array($folderId, array_map(static function (Entity $ancestor) {
 			return $ancestor->getId();
 		}, $ancestors), true)) {
-			$ancestors = array_flatten(array_map(function (Entity $ancestor) {
+			$ancestors = array_merge(...array_map(function (Entity $ancestor) {
 				return $this->findParentsOf(TreeMapper::TYPE_FOLDER, $ancestor->getId(), true);
 			}, $ancestors));
 			if (count($ancestors) === 0) {
@@ -644,7 +664,7 @@ class TreeMapper extends QBMapper {
 			$qb
 				->update('bookmarks_tree')
 				->set('parent_folder', $qb->createPositionalParameter($newParentFolderId, IQueryBuilder::PARAM_INT))
-				->set('index', $qb->createPositionalParameter($index ?? $this->countChildren($newParentFolderId)))
+				->set('index', $qb->createPositionalParameter($index ?? $this->countChildren($newParentFolderId), IQueryBuilder::PARAM_INT))
 				->where($qb->expr()->eq('id', $qb->createPositionalParameter($itemId, IQueryBuilder::PARAM_INT)))
 				->andWhere($qb->expr()->eq('type', $qb->createPositionalParameter($type)));
 			$qb->execute();
@@ -919,7 +939,12 @@ class TreeMapper extends QBMapper {
 			return $array;
 		}, $this->findChildren(TreeMapper::TYPE_FOLDER, $folderId, $isSoftDeleted));
 		$shares = array_map(function (SharedFolder $sharedFolder) use ($layers, $folderId, $isSoftDeleted) {
-			$share = $this->shareMapper->findBySharedFolder($sharedFolder->getId());
+			try {
+				$share = $this->shareMapper->findBySharedFolder($sharedFolder->getId());
+			} catch (DoesNotExistException|MultipleObjectsReturnedException|Exception $e) {
+				$this->logger->error('Failed to load a shared folder', ['exception' => $e]);
+				return null;
+			}
 			$array = $sharedFolder->toArray();
 			$array['id'] = $share->getFolderId();
 			$array['userId'] = $share->getOwner();
@@ -930,6 +955,7 @@ class TreeMapper extends QBMapper {
 			}
 			return $array;
 		}, $this->findChildren(TreeMapper::TYPE_SHARE, $folderId, $isSoftDeleted));
+		$shares = array_values(array_filter($shares, fn ($data) => $data !== null));
 		if (count($shares) > 0) {
 			array_push($folders, ...$shares);
 		}
